@@ -16,10 +16,29 @@
 ## Pesaran, M.H. (2007) A simple panel unit root test in the presence of
 ## cross-section dependence, Journal of Applied Econometrics, 22(2), pp. 265-312
 
-cipstest <- function (x, lags = 2, type = c("trend", "drift", "none"),
-                  model = c("cmg", "mg", "dmg"), truncated = FALSE, ...) {
+pseries2pdata <- function(x) {
+  ## transforms a pseries in a pdataframe with the indices as regular columns
+  ## in positions 1 and 2 (individual index, time index)
+  indices <- attr(x, "index")
+  vx <- as.numeric(x)
+  px <- cbind(indices, vx)
+  dimnames(px)[[2]] <- c("ind", "tind", deparse(substitute(x)))
+  return(pdata.frame(px, index=c("ind", "tind")))
+}
 
-  ## type=c("trend","drift","none") corresponds to Case III, II, I 
+pmerge <- function(x, y, ...) {
+  ## transf. if pseries
+  if("pseries" %in% class(x)) x <- pseries2pdata(x)
+  if("pseries" %in% class(y)) y <- pseries2pdata(y)
+  z <- merge(data.frame(x), data.frame(y), by.x=dimnames(x)[[2]][1:2],
+             by.y=dimnames(y)[[2]][1:2], ...)
+  return(z)
+}
+
+cipstest <- function (x, lags = 2, type = c("trend", "drift", "none"),
+                      model = c("cmg", "mg", "dmg"), truncated = FALSE, ...) {
+
+  ## type = c("trend", "drift", "none") corresponds to Case III, II, I 
   ## in Pesaran (2007), respectively.
 
   ## input checks
@@ -31,10 +50,10 @@ cipstest <- function (x, lags = 2, type = c("trend", "drift", "none"),
   dati <- pmerge(diff(x), lag(x))
   dati <- pmerge(dati, diff(lag(x)))
   ## minimal column names
-  clnames <- c("de","le","d1e")
+  clnames <- c("de", "le", "d1e")
   dimnames(dati)[[2]][3:5] <- clnames
-  ## add lags if lags>1
-  if(lags>1) {
+  ## add lags if lags > 1
+  if(lags > 1) {
       for(i in 2:lags) {
           dati <- pmerge(dati, diff(lag(x, i)))
           clnames <- c(clnames, paste("d", i, "e", sep=""))
@@ -49,14 +68,13 @@ cipstest <- function (x, lags = 2, type = c("trend", "drift", "none"),
                           none={"-1"})
 
   ## make formula
-  adffm <-as.formula(paste("de~le+",
-                           paste(clnames[3:(lags+2)], collapse="+"),
-                           deterministic, sep=""))
+  adffm <- as.formula(paste("de~le+",
+                            paste(clnames[3:(lags+2)], collapse="+"),
+                            deterministic, sep=""))
 
   ## estimate preliminary pooling plm, to take care of all diffs
   ## and lags in a 'panel' way (would be lost in single TS regr.s)
-  pmod <- plm(adffm, data=dati, model="pooling")
-
+    pmod <- plm(adffm, data = dati, model = "pooling")
   ## this as in pmg()
     index <- attr(model.frame(pmod), "index")
     ## group index
@@ -79,12 +97,12 @@ cipstest <- function (x, lags = 2, type = c("trend", "drift", "none"),
     k <- length(coef.names)
 
     ## CIPS test needs an ADF regression with k lags
-    ## so fm <- has to be like diff(e)~lag(e)+diff(lag(e)) etc..
+    ## so fm <- has to be like diff(e) ~ lag(e)+diff(lag(e)) etc.
 
     ## model data
     X <- model.matrix(pmod)
     y <- as.numeric(model.response(model.frame(pmod))) # remove pseries attribs
-
+    
   ## det. *minimum* group numerosity
   t <- min(tapply(X[,1],ind,length)) # TODO: == min(Ti) simpler???
 
@@ -113,7 +131,7 @@ cipstest <- function (x, lags = 2, type = c("trend", "drift", "none"),
           ## make datafr. subtracting intercept and add trend
           adfdati <- data.frame(cbind(y, X[,-1]))
           dimnames(adfdati)[[2]] <- c(clnames, "trend")
-          adffm <- update(adffm, .~.-as.numeric(tind)+trend)},
+          adffm <- update(adffm, . ~ . -as.numeric(tind) + trend)},
         drift={
           ## make df subtracting intercept
           adfdati <- data.frame(cbind(y, X[,-1]))
@@ -125,18 +143,18 @@ cipstest <- function (x, lags = 2, type = c("trend", "drift", "none"),
         )
       
       ## for each x-sect. i=1..n
-      unind<-unique(ind)
+      unind <- unique(ind)
       for(i in 1:n) {
-        tdati <- adfdati[ind==unind[i],]
+        tdati <- adfdati[ind == unind[i], ]
         tmods[[i]] <- lm(adffm, tdati)
         }
       },
     
     dmg={
       ## between-periods transformation (take means over group for each t)
-      be<-function(x,index,na.rm=TRUE) tapply(x,index,mean,na.rm=na.rm)
-      Xm<-apply(X,2,FUN=be,index=tind)[tind,]
-      ym<-apply(as.matrix(as.numeric(y)),2,FUN=be,index=tind)[tind]
+      be <- function(x,index,na.rm=TRUE) tapply(x,index,mean,na.rm=na.rm)
+      Xm <- apply(X,2,FUN=be,index=tind)[tind,]
+      ym <- apply(as.matrix(as.numeric(y)),2,FUN=be,index=tind)[tind]
       ## we do not care about demeaning the intercept or not as it is
       ## eliminated anyway
 
@@ -150,7 +168,7 @@ cipstest <- function (x, lags = 2, type = c("trend", "drift", "none"),
           ## make datafr. subtracting intercept and add trend
           adfdati <- data.frame(cbind(demy, demX[,-1]))
           dimnames(adfdati)[[2]] <- c(clnames, "trend")
-          adffm <- update(adffm, .~.-as.numeric(tind)+trend)},
+          adffm <- update(adffm, . ~ . -as.numeric(tind) + trend)},
         drift={
           ## make df subtracting intercept
           adfdati <- data.frame(cbind(demy, demX[,-1]))
@@ -162,9 +180,9 @@ cipstest <- function (x, lags = 2, type = c("trend", "drift", "none"),
 
       ## for each x-sect. i=1..n estimate (over t) a demeaned model
       ## (y_it-my_t) = alfa_i + beta_i*(X_it-mX_t) + err_it
-      unind<-unique(ind)
+      unind <- unique(ind)
       for(i in 1:n) {
-        tdati <- adfdati[ind==unind[i],]
+        tdati <- adfdati[ind == unind[i], ]
         tmods[[i]] <- lm(adffm, tdati)
         }
     },
@@ -175,16 +193,16 @@ cipstest <- function (x, lags = 2, type = c("trend", "drift", "none"),
                                drift={""},
                                none={"-1"})
       ## adjust formula
-      adffm <-as.formula(paste("de~le+",
-                         paste(clnames[3:(lags+2)], collapse="+"),
-                         "+", paste(paste(clnames, "bar", sep="."),
-                               collapse="+"),
-                         deterministic2, sep=""))
+      adffm <- as.formula(paste("de~le+",
+                           paste(clnames[3:(lags+2)], collapse = "+"),
+                           "+", paste(paste(clnames, "bar", sep = "."),
+                           collapse = "+"),
+                          deterministic2, sep = ""))
 
       ## between-periods transformation (take means over groups for each t)
-      be<-function(x,index,na.rm=TRUE) tapply(x,index,mean,na.rm=na.rm)
-      Xm<-apply(X,2,FUN=be,index=tind)[tind,]
-      ym<-apply(as.matrix(as.numeric(y)),2,FUN=be,index=tind)[tind]
+      be <- function(x,index,na.rm=TRUE) tapply(x,index,mean,na.rm=na.rm)
+      Xm <- apply(X,2,FUN=be,index=tind)[tind,]
+      ym <- apply(as.matrix(as.numeric(y)),2,FUN=be,index=tind)[tind]
 
       ## final data as dataframe, to be subsetted for single TS models
       ## (purge intercepts etc., if 'trend' fix this variable's name)
@@ -197,16 +215,16 @@ cipstest <- function (x, lags = 2, type = c("trend", "drift", "none"),
           adfdati <- adfdati[,-(dim(adfdati)[[2]])]
           dimnames(adfdati)[[2]] <- c(clnames, "trend",
                                       paste(clnames, "bar", sep="."))
-          adffm <- update(adffm, .~.-as.numeric(tind)+trend)},
+          adffm <- update(adffm, . ~ . -as.numeric(tind) + trend)},
         
         drift={
-          augX<-cbind(X[,-1], ym, Xm[,-1])
+          augX <- cbind(X[,-1], ym, Xm[,-1])
           adfdati <- data.frame(cbind(y, augX))
           dimnames(adfdati)[[2]] <- c(clnames,
                                       paste(clnames, "bar", sep="."))},
         none={
           ## no intercepts here
-          augX<-cbind(X,ym,Xm)
+          augX <- cbind(X,ym,Xm)
           adfdati <- data.frame(cbind(y, augX))
           dimnames(adfdati)[[2]] <- c(clnames,
                                       paste(clnames, "bar", sep="."))
@@ -216,8 +234,8 @@ cipstest <- function (x, lags = 2, type = c("trend", "drift", "none"),
       ## y_it = alfa_i + beta_i*X_it + c1_i*my_t + c2_i*mX_t + err_it
       unind<-unique(ind)
       for(i in 1:n) {
-        tdati <- adfdati[ind==unind[i],]
-        tmods[[i]]<-lm(adffm, tdati)
+        tdati <- adfdati[ind == unind[i], ]
+        tmods[[i]] <- lm(adffm, tdati)
         }
   })
 
@@ -229,21 +247,21 @@ cipstest <- function (x, lags = 2, type = c("trend", "drift", "none"),
   
   if(truncated) {
       ## set bounds, Pesaran (2007), p. 277
-        ## NB: there is a  typo in the paper:
+        ## NB: there is a  typo in the paper (see p. 279/281 to confirm):
         ##   Case I: "with an intercept or trend" -> "with_out_ an intercept or trend"
         ## "with_out_ an intercept or trend (Case I): K1 = 6.12, K2 = 4.16"
         ## "with an intercept and no trend (Case II): K1 = 6.19, K2 = 2.61"
         ## "with a linear trend (Case III):           K1 = 6.42, K2 = 1.70"
         ## (use negative values for K1's to ease assignment if bound is reached)
       trbounds <- switch(match.arg(type),
-                          trend = {c(-6.12, 4.16)}, ## TODO?
+                          none  = {c(-6.12, 4.16)},
                           drift = {c(-6.19, 2.61)},
-                          none  = {c(-6.42, 1.70)}) ## TODO?
+                          trend = {c(-6.42, 1.70)})
       ## formulae (34) in Pesaran (2007):
       ## truncate at lower bound 
-      tstats <- ifelse(tstats>trbounds[1], tstats, trbounds[1])
+      tstats <- ifelse(tstats > trbounds[1], tstats, trbounds[1])
       ## truncate at upper bound
-      tstats <- ifelse(tstats<trbounds[2], tstats, trbounds[2])
+      tstats <- ifelse(tstats < trbounds[2], tstats, trbounds[2])
   }
 
   ## here allow for '...' to pass 'na.rm=TRUE' in case (but see what happens
@@ -253,10 +271,10 @@ cipstest <- function (x, lags = 2, type = c("trend", "drift", "none"),
 
   ## if pval out of critical values' then set at boundary and issue
   ## a warning
-  if(pval=="> 0.10") {
+  if(pval == "> 0.10") {
       pval <- 0.10
       warning("p-value greater than printed p-value")
-  } else if(pval=="< 0.01") {
+  } else if(pval == "< 0.01") {
       pval <- 0.01
       warning("p-value smaller than printed p-value")
   }
@@ -282,7 +300,7 @@ cipstest <- function (x, lags = 2, type = c("trend", "drift", "none"),
 critvals <- function(stat, n, T., type=c("trend", "drift", "none"),
                      truncated = FALSE) {
     ## auxiliary function for cipstest()
-    ## extracts --or calculates by interpolation-- p.values for the
+    ## extracts --or calculates by interpolation-- p-values for the
     ## (averaged) CIPS statistic depending on whether n and T,
     ## given the critical values of average of individual cross-sectionally
     ## augmented Dickey-Fuller distribution
@@ -509,10 +527,10 @@ cvprox <- approx(cv, c(0.01,0.05,0.1), n=200)
 cvseq <- cvprox$x
 pvseq <- cvprox$y
 
-if(stat<min(cv)) {
+if(stat < min(cv)) {
     pval <- "< 0.01"
 } else {
-    if(stat>max(cv)) {
+    if(stat > max(cv)) {
         pval <- "> 0.10"
         } else {
             if(stat %in% cv) {
