@@ -30,18 +30,18 @@ bdiag <- function(...){
 
 
 twosls <- function(y, X, W, intercept = FALSE){
-  Xhat <- lm(X ~ W)$fit
+  Xhat <- lm(X ~ W)$fitted.values
   if(!is.matrix(Xhat)){
     Xhat <- matrix(Xhat, ncol = 1)
     colnames(Xhat) <- colnames(X)
   }
   if(intercept){
-    model <- lm(y~Xhat)
-    yhat <- as.vector(crossprod(t(cbind(1,X)),coef(model)))
+    model <- lm(y ~ Xhat)
+    yhat <- as.vector(crossprod(t(cbind(1, X)), coef(model)))
   }
   else{
-    model <- lm(y~Xhat-1)
-    yhat <- as.vector(crossprod(t(X),coef(model)))
+    model <- lm(y ~ Xhat - 1)
+    yhat <- as.vector(crossprod(t(X), coef(model)))
   }
   model$residuals <- y - yhat
   model
@@ -57,6 +57,14 @@ sumres <- function(x){
   sr
 }
 
+sumsq <- function(x, ...){
+    xb <- mean(x, na.rm = TRUE)
+    sum((na.omit(x)-xb)^2)
+}
+
+
+
+# suml(x) is replaced by Reduce("+", x)
 ## suml <- function(x){
 ##   n <- length(x)
 ##   if (!is.null(dim(x[[1]]))){
@@ -103,7 +111,8 @@ rbindl <- function(x){
 }
 
 expand.formula <- function(x){
-  if (!any(class(x) == "Formula")) stop("not a Formula object")
+  oclass <- class(x)
+  if (! any(class(x) == "Formula")) stop("not a Formula object")
   if (length(x)[2] != 2) stop("not a two part formula")
   xs <- structure(x, class = "formula")
   has.response <- attr(terms(xs),"response") == 1
@@ -118,25 +127,25 @@ expand.formula <- function(x){
   firstpart <- rhs[[2]]
   secondpart <- rhs[[3]]
   if (has.response){
-    one <- do.call("~",list(y,firstpart))
-    two <- do.call("~",list(y,secondpart))
+    one <- do.call("~", list(y,firstpart))
+    two <- do.call("~", list(y,secondpart))
   }
   else{
-    one <- do.call("~",list(firstpart))
-    two <- do.call("~",list(secondpart))
+    one <- do.call("~", list(firstpart))
+    two <- do.call("~", list(secondpart))
   }
-  two <- update(one,two)
+  two <- update(one, two)
   one <- paste(deparse(one), collapse = "")
   two <- paste(deparse(two[[3]]), collapse = "")
   result <- as.formula(paste(one, "|", two, collapse = ""));
   result <- as.Formula(result)
-  class(result) <- c("pFormula", class(result))
-  result
+  #YC  class(result) <- c("pFormula", class(result))
+  structure(result, class = oclass)
 }
 
 
 lev2var <- function(x, ...){
-  # take a data.frame and returns a vector of variable names, the
+  # takes a data.frame and returns a vector of variable names, the
   # names of the vector being the names of the effect
   
   is.fact <- sapply(x, is.factor)
@@ -177,7 +186,7 @@ has.intercept.formula <- function(object, ...) {
 }
 
 has.intercept.Formula <- function(object, rhs = NULL, ...) {
-  ## NOTE: return a logical vector of the necessary length
+  ## NOTE: returns a logical vector of the necessary length
   ## (which might be > 1)
   if(is.null(rhs)) rhs <- 1:length(attr(object, "rhs"))
   sapply(rhs, function(x) has.intercept(formula(object, lhs = 0, rhs = x)))
@@ -198,8 +207,8 @@ pres <- function(x) {  # pres.panelmodel
   ## extracts model residuals as pseries
 
   ## extract indices
-  groupind <-attr(x$model, "index")[,1]   #attr(model.frame(x), "index")[[1]]
-  timeind  <-attr(x$model, "index")[,2]   #attr(model.frame(x), "index")[[2]]
+  groupind <-attr(x$model, "index")[,1]
+  timeind  <-attr(x$model, "index")[,2]
   
   # fix to allow operation with pggls, pmg (NB: pmg? meant: plm?)
   # [NB: one day, make this cleaner; with the describe framework?]
@@ -210,16 +219,15 @@ pres <- function(x) {  # pres.panelmodel
   if(exists("maybe_fd") && maybe_fd == "fd") {
     groupi <- as.numeric(groupind)
     ## make vector =1 on first obs in each group, 0 elsewhere
-    selector <-groupi-c(0,groupi[-length(groupi)])
+    selector <- groupi - c(0, groupi[-length(groupi)])
     selector[1] <- 1 # the first must always be 1
     ## eliminate first obs in time for each group
     groupind <- groupind[!selector]
     timeind <- timeind[!selector]
   }
 
-  resdata <- data.frame(ee=x$residuals, ind=groupind, tind=timeind)
-  pee <- pdata.frame(resdata, index=c("ind","tind"))
-
+  resdata <- data.frame(ee = x$residuals, ind = groupind, tind = timeind)
+  pee <- pdata.frame(resdata, index = c("ind", "tind"))
   pres <- pee$ee
   return(pres)
 }
@@ -238,25 +246,51 @@ nobs.panelmodel <- function(object, ...) {
 # No of obs calculated as in print.summary.pgmm [code copied from there]
 nobs.pgmm <- function(object, ...) {
   if (inherits(object, "pgmm")) return(sum(unlist(object$residuals) != 0))
-    else stop("Input 'object' needs to be of class 'pgmm', i. e. a GMM estimation with panel data estimated by pgmm()")
+    else stop("Input 'object' needs to be of class 'pgmm', i. e., a GMM estimation with panel data estimated by pgmm()")
 }
 
 
 
 # punbalancedness: measures for unbalancedness of a pandel data set
 # as defined in Ahrens/Pincus (1981), p. 228 (gamma and nu)
+# and for nested panel structures as in Baltagi/Song/Jung (2001), pp. 368-369
 punbalancedness.default <- function(x, ...) {
-  pdim <- pdim(x, ...)
+
+  ii <- index(x)
   
-  N <- pdim$nT$n # no. of individuals
-  Totalobs <- pdim$nT$N # no. of total observations
-  Ti <- pdim$Tint$Ti
-  Tavg <- sum(Ti)/N
-  
-  r1 <- N / (Tavg * sum(1/Ti))
-  r2 <- 1 / (N * (sum( (Ti/Totalobs)^2)))
-  
-  return(c(gamma = r1, nu = r2))
+  if (length(ii) == 2) {
+   ## original Ahrens/Pincus (1981)
+    pdim <- pdim(x, ...)
+    N <- pdim$nT$n # no. of individuals
+    Totalobs <- pdim$nT$N # no. of total observations
+    Ti <- pdim$Tint$Ti
+    Tavg <- sum(Ti)/N
+    
+    r1 <- N / (Tavg * sum(1/Ti))
+    r2 <- 1 / (N * (sum( (Ti/Totalobs)^2)))
+    result <- c(gamma = r1, nu = r2)
+  } else {
+    if (length(ii) == 3) {
+     ## extension to nested model with additional group variable
+     ## Baltagi/Song/Jung (2001), pp. 368-369
+      ids <- ii[[1]]
+      tss <- ii[[2]]
+      gps <- ii[[3]]
+      Tis <- unique(data.frame(tss, gps))
+      Tis <- table(Tis$gps)               # no of max time periods per group
+      Nis <- unique(data.frame(ids, gps))
+      Nis <- table(Nis$gps)               # no of individuals per group
+      M <- length(unique(gps))            # no of unique groups
+      Nbar <- sum(Nis)/M
+      Tbar <- sum(Tis)/M
+      
+      c1 <- M / (Nbar * sum(1/Nis))
+      c2 <- M / (Tbar * sum(1/Tis))
+      c3 <- M / (sum(Nis * Tis)/M * sum(1/(Nis*Tis)))
+      result <- (c(c1 = c1, c2 = c2, c3 = c3))
+    } else stop(paste0("unsupported number of dimensions: ", length(ii)))
+  }
+  return(result)
 }
 
 punbalancedness.pdata.frame <- function(x, ...) {
@@ -277,19 +311,22 @@ punbalancedness <- function(x, ...) {
 }
 
 
-# helper function
+# helper function, used in pdata.frame()
 fancy.row.names <- function(index, sep = "-") {
-  return(paste(index[[1]], index[[2]], sep = sep))
+  if (length(index) == 2) {result <- paste(index[[1]], index[[2]], sep = sep)}
+  # this in the order also used for sorting (group, id, time):
+  if (length(index) == 3) {result <- paste(index[[3]], index[[1]], index[[2]], sep = sep)}
+  return(result)
 }
 
-# helper function for pwaldtest: trans_clubSandwich_vcov
-# translate clubSandwich's vcov object so it is suitable for summary.plm, plm's pwaldtest
-# attribute "cluster" in clubSandwich's vcov objects contains the cluster variable it
-# plm's vcov objects also have attribute "cluster" but it contains a character as
+# trans_clubSandwich_vcov: helper function for pwaldtest()
+# translate vcov object from package clubSandwich so it is suitable for summary.plm, plm's pwaldtest.
+# Attribute "cluster" in clubSandwich's vcov objects contains the cluster variable itself.
+# plm's vcov object also has attribute "cluster" but it contains a character as
 # information about the cluster dimension (either "group" or "time")
 #
 # inputs:
-#   * CSvcov: a vcov as returned by clubSandwich's vcovCR function [class("vcovCR", "clubSandwich")]
+#   * CSvcov: a vcov as returned by clubSandwich's vcovCR function [class c("vcovCR", "clubSandwich")]
 #   * index: the index belonging to a plm object/model
 # return value:
 #   * modified CSvcov (substituted attribute "cluster" with suitable character or NULL)
@@ -312,7 +349,7 @@ trans_clubSandwich_vcov <- function(CSvcov, index) {
   return(CSvcov)
 }
 
-## helper function to extract one or more t value(s) (coef/s.e.) for a coefficient from model object
+## gettvalue: helper function to extract one or more t value(s) (coef/s.e.) for a coefficient from model object
 ## useful if one wants to avoid the computation of a whole lot of values with summary()
 gettvalue <- function(x, coefname) {
   # x: model object (usually class plm or lm)
@@ -324,10 +361,10 @@ gettvalue <- function(x, coefname) {
   return(tvalue)
 }
 
-# helper function: "p-function" for mixed chisq (also called chi-bar-squared)
+# pchibarsq: helper function: "p-function" for mixed chisq (also called chi-bar-squared)
 # used in plmtest(., type = "ghm"), see Baltagi (2013), pp. 71-72, 74, 88, 202-203, 209
 #
-# a reference seems to be
+# a reference for the distribution seems to be
 # Dykstra, R./El Barmi, H., Chi-Bar-Square Distributions, in: Encyclopedia of Statistical Sciences, 
 # DOI: 10.1002/0471667196.ess0265.pub2
 pchibarsq <- function(q, df, weights, lower.tail = TRUE, ... ) {
@@ -335,3 +372,41 @@ pchibarsq <- function(q, df, weights, lower.tail = TRUE, ... ) {
   res <- sum(weights * pchisq(q, df = df, lower.tail = lower.tail, ...))
   return(res)
 }
+
+## TODO: is print.form actually used? - delete?
+print.form <- function(x, length.line){
+  x <- deparse(x,width.cutoff=length.line)
+  n <- length(x)
+  cat(paste(x[1],"\n",sep=""))
+  if (n>1){
+    for (i in 2:n){
+      cat(paste(x[i],"\n",sep=""))
+    }
+  }
+}
+
+# check_propagation_correct_class: helper function
+# Function checks if the class and storage mode (type) of an object match 
+# and corrects its class attribute if not
+#
+# A mismatch can occur if a pseries of lower class and type logical or interger
+# are propagated to higher type by an arithmetic operation as R's arithmetic
+# operations do not change the first value of class attribute for
+# c("pseries", "logical/integer").
+# E.g.
+#  x <- c(1L, 2L, 3L)
+#  x + 1.5
+# results in class propagation from class "integer" to "numeric" 
+# but not if x is of class c("pseries", "interger")
+check_propagation_correct_class <- function(x) {
+  # x: a pseries object (usually)
+  if (any((pos <- inherits(x, c("logical" ,"integer", "numeric"), which = TRUE)) > 0)) {
+    pos <- pos[pos > 0] # non-matches in inherits(..., which = TRUE) results in 0
+    switch(typeof(x),
+           "double"  = { attr(x, "class")[pos] <- "numeric" },
+           "integer" = { attr(x, "class")[pos] <- "integer" },
+           "complex" = { attr(x, "class")[pos] <- "complex" }
+    )
+  }
+  return(x)
+}
